/*
 * drivers/media/video/mt9p031.c
 *
 * Based on TI TVP5146/47 decoder driver
 *
 *
 * This package is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "linux/i2c.h"
#include "linux/delay.h"
#include "linux/videodev2.h"
#include "linux/io.h"

#include "media/v4l2-int-device.h"
#include "media/mt9p031.h"

#include "mt9p031_regs.h"

#define I2C_RETRY_COUNT                 (5)
#define MT9P031_MODULE_NAME             "mt9p031"

/* Debug functions */
static int debug = 1;
module_param(debug, bool, 0644);
MODULE_PARM_DESC(debug, "Debug level (0-1)");

/*
 * enum mt9p031_std - enum for supported standards
 */
enum mt9p031_std {
	MT9P031_STD_VGA =      0,
	MT9P031_STD_XGA		,
	MT9P031_STD_720P	,
	MT9P031_STD_1080P 	,
	MT9P031_STD_2592X1944 	,
	MT9P031_STD_INVALID
};

/*
 * enum mt9p031_state - enum for different decoder states
 */
enum mt9p031_state {
	STATE_NOT_DETECTED,
	STATE_DETECTED
};

/*
 * struct mt9p031_std_info - Structure to store standard informations
 * @width: Line width in pixels
 * @height:Number of active lines
 * @video_std: Value to write in REG_VIDEO_STD register
 * @standard: v4l2 standard structure information
 */
struct mt9p031_std_info {
	unsigned long width;
	unsigned long height;
	u8 video_std;
	struct v4l2_standard standard;
};

/*
 * struct mt9p031_decoded - decoder object
 * @v4l2_int_device: Slave handle
 * @pdata: Board specific
 * @client: I2C client data
 * @id: Entry from I2C table
 * @ver: Chip version
 * @state: decoder state - detected or not-detected
 * @pix: Current pixel format
 * @num_fmts: Number of formats
 * @fmt_list: Format list
 * @current_std: Current standard
 * @num_stds: Number of standards
 * @std_list: Standards list
 */
struct mt9p031_decoder {
	struct v4l2_int_device *v4l2_int_device;
	const struct mt9p031_platform_data *pdata;
	struct i2c_client *client;

	struct i2c_device_id *id;

	int ver;
	enum mt9p031_state state;

	struct v4l2_pix_format pix;
	int num_fmts;
	const struct v4l2_fmtdesc *fmt_list;

	enum mt9p031_std current_std;
	int num_stds;
	struct mt9p031_std_info *std_list;

	int current_gain;
	int current_exposure;
};

/*These are the registers needed for all the different configurations*/
static struct mt9p031_reg mt9p031_vga_reg[] = {
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, MT9P031_HALT_MODE},
	{TOK_WRITE, MT9P031_WIDTH, 2559},
	{TOK_WRITE, MT9P031_HEIGHT, 1919},
	{TOK_WRITE, MT9P031_SHUTTER_WIDTH,1000 & MT9P031_SHUTTER_WIDTH_LOWER_MASK},
	{TOK_WRITE, MT9P031_SHUTTER_WIDTH_UPPER,1000 >> MT9P031_SHUTTER_WIDTH_UPPER_SHIFT},
	{TOK_WRITE, MT9P031_ROW_ADDR_MODE, 0x33},
	{TOK_WRITE, MT9P031_COL_ADDR_MODE, 0x33},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_POWER_UP},
	{TOK_WRITE, MT9P031_PLL_CFG1, ((24 << MT9P031_PLL_M_SHIFT) | (2 & MT9P031_PLL_N_MASK))},
	{TOK_WRITE, MT9P031_PLL_CFG2, (3 & MT9P031_PLL_P1_MASK)},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_USE_PLL_CLK},
	{TOK_WRITE, MT9P031_GLOBAL_GAIN , 316},
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, 0X1F8E & MT9P031_OUTPUT_CTRL_MASK},
	{TOK_TERM, 0, 0}
};


static struct mt9p031_reg mt9p031_xga_reg[] = {
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, MT9P031_HALT_MODE},
	{TOK_WRITE, MT9P031_HEIGHT, 1535},
	{TOK_WRITE, MT9P031_WIDTH, 2047},
	{TOK_WRITE, MT9P031_VBLANK, 8},
	{TOK_WRITE, MT9P031_HBLANK, 0},
	{TOK_WRITE, MT9P031_SHUTTER_WIDTH,1000 & MT9P031_SHUTTER_WIDTH_LOWER_MASK},
	{TOK_WRITE, MT9P031_SHUTTER_WIDTH_UPPER,1000 >> MT9P031_SHUTTER_WIDTH_UPPER_SHIFT},
	{TOK_WRITE, MT9P031_ROW_ADDR_MODE, 0x11},
	{TOK_WRITE, MT9P031_COL_ADDR_MODE, 0x11},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_POWER_UP},
	{TOK_WRITE, MT9P031_PLL_CFG1, ((255 << MT9P031_PLL_M_SHIFT) | (24 & MT9P031_PLL_N_MASK))},
	{TOK_WRITE, MT9P031_PLL_CFG2, (3 & MT9P031_PLL_P1_MASK)},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_USE_PLL_CLK},
	{TOK_WRITE, MT9P031_GLOBAL_GAIN , 316},
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, 0X1F8E & MT9P031_OUTPUT_CTRL_MASK},
	{TOK_TERM, 0, 0}
};

static struct mt9p031_reg mt9p031_720p_reg[] = {
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, MT9P031_HALT_MODE},
	{TOK_WRITE, MT9P031_HEIGHT, 1439},
	{TOK_WRITE, MT9P031_WIDTH, 2559},
	{TOK_WRITE, MT9P031_VBLANK, 0},
	{TOK_WRITE, MT9P031_HBLANK, 0},
	{TOK_WRITE, MT9P031_SHUTTER_WIDTH,1000 & MT9P031_SHUTTER_WIDTH_LOWER_MASK},
	{TOK_WRITE, MT9P031_SHUTTER_WIDTH_UPPER,1000 >> MT9P031_SHUTTER_WIDTH_UPPER_SHIFT},
	{TOK_WRITE, MT9P031_ROW_ADDR_MODE, 0x11},
	{TOK_WRITE, MT9P031_COL_ADDR_MODE, 0x11},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_POWER_UP},
	{TOK_WRITE, MT9P031_PLL_CFG1, ((255 << MT9P031_PLL_M_SHIFT) | (23 & MT9P031_PLL_N_MASK))},
	{TOK_WRITE, MT9P031_PLL_CFG2, (3 & MT9P031_PLL_P1_MASK)},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_USE_PLL_CLK},
	{TOK_WRITE, MT9P031_GLOBAL_GAIN , 316},
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, 0X1F8E & MT9P031_OUTPUT_CTRL_MASK},
	{TOK_TERM, 0, 0}
};

static struct mt9p031_reg mt9p031_1080p_reg[] = {
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, MT9P031_HALT_MODE},
	{TOK_WRITE, MT9P031_HEIGHT, 1079},
	{TOK_WRITE, MT9P031_WIDTH, 1919},
	{TOK_WRITE, MT9P031_VBLANK, 3},
	{TOK_WRITE, MT9P031_HBLANK, 0},
	{TOK_WRITE, MT9P031_SHUTTER_WIDTH,1000 & MT9P031_SHUTTER_WIDTH_LOWER_MASK},
	{TOK_WRITE, MT9P031_SHUTTER_WIDTH_UPPER,1000 >> MT9P031_SHUTTER_WIDTH_UPPER_SHIFT},
	{TOK_WRITE, MT9P031_ROW_ADDR_MODE, 0x0},
	{TOK_WRITE, MT9P031_COL_ADDR_MODE, 0x0},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_POWER_UP},
	{TOK_WRITE, MT9P031_PLL_CFG1, ((255 << MT9P031_PLL_M_SHIFT) | (25 & MT9P031_PLL_N_MASK))},
	{TOK_WRITE, MT9P031_PLL_CFG2, (3 & MT9P031_PLL_P1_MASK)},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_USE_PLL_CLK},
	{TOK_WRITE, MT9P031_GLOBAL_GAIN , 316},
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, 0X1F8E & MT9P031_OUTPUT_CTRL_MASK},
	{TOK_TERM, 0, 0}
};

static struct mt9p031_reg mt9p031_2592X1944_reg[] = {
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, MT9P031_HALT_MODE},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_POWER_UP},
	{TOK_WRITE, MT9P031_PLL_CFG1, ((24 << MT9P031_PLL_M_SHIFT) | (2 & MT9P031_PLL_N_MASK))},
	{TOK_WRITE, MT9P031_PLL_CFG2, (3 & MT9P031_PLL_P1_MASK)},
	{TOK_WRITE, MT9P031_PLL_CTRL, MT9P031_PLL_CTRL_USE_PLL_CLK},
	{TOK_DELAY, 0, 100},
	{TOK_WRITE, MT9P031_GLOBAL_GAIN , 316},
	{TOK_WRITE, MT9P031_OUTPUT_CTRL, 0X1F8E & MT9P031_OUTPUT_CTRL_MASK},
	{TOK_TERM, 0, 0}
};


/* MT9P031 default register values */
static struct mt9p031_reg mt9p031_reg_list[] = {
	{TOK_WRITE, MT9P031_RESTART, 0x1},
	{TOK_DELAY, 0, 100},
	{TOK_WRITE, MT9P031_RESTART, 0x0},
	{TOK_WRITE, MT9P031_PIXEL_CLK_CTRL, 0x8000},
	{TOK_WRITE, MT9P031_GLOBAL_GAIN , 316},
	{TOK_WRITE, 0x29, 0X481},
	{TOK_WRITE, 0X3E, 0X87},
	{TOK_WRITE, 0X3f, 0X07},
	{TOK_WRITE, 0x41, 0X0003},
	{TOK_WRITE, 0X42, 0X0003},
	{TOK_WRITE, 0X43, 0x0003},
	{TOK_WRITE, 0x48, 0X18},
	{TOK_WRITE, 0X5F, 0X1C16},
	{TOK_WRITE, 0X57, 0X7},
	{TOK_WRITE, 0x70, 0X005C},
	{TOK_WRITE, 0X71, 0X5B00},
	{TOK_WRITE, 0X72, 0X5900},
	{TOK_WRITE, 0x73, 0X200},
	{TOK_WRITE, 0X74, 0X200},
	{TOK_WRITE, 0X75, 0X2800},
	{TOK_WRITE, 0x76, 0X3E29},
	{TOK_WRITE, 0X77, 0X3E29},
	{TOK_WRITE, 0X78, 0X583F},
	{TOK_WRITE, 0x79, 0X5B00},
	{TOK_WRITE, 0X7A, 0X5A00},
	{TOK_WRITE, 0X7B, 0X5900},
	{TOK_WRITE, 0x7C, 0X5900},
	{TOK_WRITE, 0X7E, 0X5900},
	{TOK_WRITE, 0X7F, 0X5900},
	{TOK_TERM, 0, 0}
};

/* List of image formats supported by mt9p031
 * Currently we are using 8 bit mode only, but can be
 * extended to 10/20 bit mode.
 */
static const struct v4l2_fmtdesc mt9p031_fmt_list[] = {
	{
	 .index = 0,
	 .type = V4L2_BUF_TYPE_VIDEO_CAPTURE,
	 .flags = 0,
	 .description = "10-Bit RGB Format",
	 .pixelformat = V4L2_PIX_FMT_SGRBG10,
	},
};

/*
 * Supported standards -
 *
 * Currently supports two standards only, need to add support for rest of the
 * modes, like SECAM, etc...
 */
static struct mt9p031_std_info mt9p031_std_list[] = {
	[MT9P031_STD_VGA] = {
	 .width = 640,
	 .height = 480,
	 .video_std = MT9P031_IMAGE_STD_VGA,
	 .standard = {
		      .index = MT9P031_STD_VGA,
		      .id = MT9P031_IMAGE_STD_VGA,
		      .name = "VGA",
		      .frameperiod = {1, 13},
		      .framelines = 480
		     },
	},
	[MT9P031_STD_XGA] = {
	 .width = 1024,
	 .height = 768,
	 .video_std = MT9P031_IMAGE_STD_XGA,
	 .standard = {
		      .index = MT9P031_STD_XGA,
		      .id = MT9P031_IMAGE_STD_XGA,
		      .name = "XGA",
		      .frameperiod = {1, 30},
		      .framelines = 768
		     },
	},
	[MT9P031_STD_720P] = {
	 .width = 1280,
	 .height = 720,
	 .video_std = MT9P031_IMAGE_STD_720P,
	 .standard = {
		      .index = MT9P031_STD_720P,
		      .id = MT9P031_IMAGE_STD_720P,
		      .name = "720P",
		      .frameperiod = {1, 30},
		      .framelines = 720
		     },
	},
	[MT9P031_STD_1080P] = {
	 .width = 1920,
	 .height = 1080,
	 .video_std = MT9P031_IMAGE_STD_1080P,
	 .standard = {
		      .index = MT9P031_STD_1080P,
		      .id = MT9P031_IMAGE_STD_1080P,
		      .name = "1080P",
		      .frameperiod = {1, 15},
		      .framelines = 1080
		     },
	},
	[MT9P031_STD_2592X1944] = {
	 .width = 2592,
	 .height = 1944,
	 .video_std = MT9P031_IMAGE_STD_2592X1944,
	 .standard = {
		      .index = MT9P031_STD_2592X1944,
		      .id = MT9P031_IMAGE_STD_2592X1944,
		      .name = "2592x1944 Max Resolution",
		      .frameperiod = {1, 7},
		      .framelines = 1944
		     },
	},
};
/*
 * Control structure for Auto Gain
 *     This is temporary data, will get replaced once
 *     v4l2_ctrl_query_fill supports it.
 */
static const struct v4l2_queryctrl mt9p031_autogain_ctrl = {
	.id = V4L2_CID_AUTOGAIN,
	.name = "Gain, Automatic",
	.type = V4L2_CTRL_TYPE_BOOLEAN,
	.minimum = 0,
	.maximum = 1,
	.step = 1,
	.default_value = 1,
};

const struct v4l2_fract mt9p031_frameintervals[] = {
	{  .numerator = 1, .denominator = 30 }
};

/*
 * Gain table for  m9p031
 * Gain is not completely linear as we give more weight to lower
 * digital gain than greatest possible linearity of gain
 */
static struct mt9p031_gain {
	u32 analog;
	u32 analog_mult;
	u32 digital;
} const mt9p031_gain_table[] = {
	{   8,  0,  0}, /* 1.00 */
	{  10,  0,  0}, /* 1.25 */
	{  12,  0,  0}, /* 1.50 */
	{  14,  0,  0}, /* 1.75 */
	{   8,  1,  0}, /* 2.00 */
	{  18,  0,  0}, /* 2.25 */
	{  10,  1,  0}, /* 2.50 */
	{  11,  1,  0}, /* 2.75 */
	{  24,  0,  0}, /* 3.00 */
	{  26,  0,  0}, /* 3.25 */
	{  28,  0,  0}, /* 3.50 */
	{  30,  0,  0}, /* 3.75 */
	{  32,  0,  0}, /* 4.00 */
	{  34,  0,  0}, /* 4.25 */
	{  36,  0,  0}, /* 4.50 */
	{  38,  0,  0}, /* 4.75 */
	{  40,  0,  0}, /* 5.00 */
	{  42,  0,  0}, /* 5.25 */
	{  44,  0,  0}, /* 5.50 */
	{  46,  0,  0}, /* 5.75 */
	{  48,  0,  0}, /* 6.00 */
	{  50,  0,  0}, /* 6.25 */
	{  51,  0,  0}, /* 6.38 */
	{  53,  0,  0}, /* 6.62 */
	{  55,  0,  0}, /* 6.88 */
	{  57,  0,  0}, /* 7.12 */
	{  59,  0,  0}, /* 7.38 */
	{  61,  0,  0}, /* 7.62 */
	{  63,  0,  0}, /* 7.88 */
	{  58,  0,  1}, /* 8.16 */
	{  60,  0,  1}, /* 8.44 */
	{  62,  0,  1}, /* 8.72 */
	{  63,  0,  1}, /* 8.86 */
	{  59,  0,  2}, /* 9.22 */
	{  60,  0,  2}, /* 9.38 */
	{  62,  0,  2}, /* 9.69 */
	{  63,  0,  2}, /* 9.84 */
	{  59,  0,  3}, /* 10.14 */
	{  60,  0,  3}, /* 10.31 */
	{  62,  0,  3}, /* 10.66 */
	{  63,  0,  3}, /* 10.83 */
	{  59,  0,  4}, /* 11.06 */
	{  61,  0,  4}, /* 11.44 */
	{  62,  0,  4}, /* 11.62 */
	{  63,  0,  4}, /* 11.81 */
	{  60,  0,  5}, /* 12.19 */
	{  61,  0,  5}, /* 12.39 */
	{  62,  0,  5}, /* 12.59 */
	{  63,  0,  5}, /* 12.80 */
	{  60,  0,  6}, /* 13.12 */
	{  61,  0,  6}, /* 13.34 */
	{  62,  0,  6}, /* 13.56 */
	{  63,  0,  6}, /* 13.78 */
	{  60,  0,  7}, /* 14.06 */
	{  61,  0,  7}, /* 14.30 */
	{  62,  0,  7}, /* 14.53 */
	{  63,  0,  7}, /* 14.77 */
	{  60,  1,  0}, /* 15.00 */
	{  61,  1,  0}, /* 15.25 */
	{  62,  1,  0}, /* 15.50 */
	{  63,  1,  0}, /* 15.75 */
	{  57,  1,  1}, /* 16.03 */
	{  58,  1,  1}, /* 16.31 */
	{  59,  1,  1}, /* 16.59 */
	{  60,  1,  1}, /* 16.88 */
	{  61,  1,  1}, /* 17.16 */
	{  62,  1,  1}, /* 17.44 */
	{  63,  1,  1}, /* 17.72 */
	{  57,  1,  2}, /* 17.81 */
	{  61,  0, 11}, /* 18.11 */
	{  62,  0, 11}, /* 18.41 */
	{  59,  1,  2}, /* 18.44 */
	{  60,  1,  2}, /* 18.75 */
	{  61,  1,  2}, /* 19.06 */
	{  62,  1,  2}, /* 19.38 */
	{  63,  1,  2}, /* 19.69 */
	{  58,  1,  3}, /* 19.94 */
	{  61,  0, 13}, /* 20.02 */
	{  59,  1,  3}, /* 20.28 */
	{  60,  1,  3}, /* 20.62 */
	{  63,  1,  3}, /* 21.66 */
	{  58,  1,  4}, /* 21.75 */
	{  61,  0, 15}, /* 21.92 */
	{  59,  1,  4}, /* 22.12 */
	{  62,  0, 15}, /* 22.28 */
	{  60,  1,  4}, /* 22.50 */
	{  63,  0, 15}, /* 22.64 */
	{  61,  1,  4}, /* 22.88 */
	{  62,  1,  4}, /* 23.25 */
	{  63,  1,  4}, /* 23.62 */
	{  61,  0, 17}, /* 23.83 */
	{  59,  1,  5}, /* 23.97 */
	{  62,  0, 17}, /* 24.22 */
	{  60,  1,  5}, /* 24.38 */
	{  63,  0, 17}, /* 24.61 */
	{  61,  1,  5}, /* 24.78 */
	{  62,  1,  5}, /* 25.19 */
	{  63,  1,  5}, /* 25.59 */
	{  61,  0, 19}, /* 25.73 */
	{  59,  1,  6}, /* 25.81 */
	{  62,  0, 19}, /* 26.16 */
	{  60,  1,  6}, /* 26.25 */
	{  63,  0, 19}, /* 26.58 */
	{  61,  1,  6}, /* 26.69 */
	{  62,  1,  6}, /* 27.12 */
	{  63,  1,  6}, /* 27.56 */
	{  61,  0, 21}, /* 27.64 */
	{  59,  1,  7}, /* 27.66 */
	{  62,  0, 21}, /* 28.09 */
	{  60,  1,  7}, /* 28.12 */
	{  63,  0, 21}, /* 28.55 */
	{  61,  1,  7}, /* 28.59 */
	{  62,  1,  7}, /* 29.06 */
	{  63,  1,  7}, /* 29.53 */
	{  61,  0, 23}, /* 29.55 */
	{  60,  1,  8}, /* 30.00 */
	{  62,  0, 23}, /* 30.03 */
	{  61,  1,  8}, /* 30.50 */
	{  63,  0, 23}, /* 30.52 */
	{  62,  1,  8}, /* 31.00 */
	{  63,  1,  8}, /* 31.50 */
	{  58,  0, 27}, /* 31.72 */
	{  60,  1,  9}, /* 31.88 */
	{  62,  0, 25}, /* 31.97 */
	{  61,  1,  9}, /* 32.41 */
	{  63,  0, 25}, /* 32.48 */
	{  62,  1,  9}, /* 32.94 */
	{  59,  1, 10}, /* 33.19 */
	{  63,  1,  9}, /* 33.47 */
	{  60,  1, 10}, /* 33.75 */
	{  57,  1, 11}, /* 33.84 */
	{  62,  0, 27}, /* 33.91 */
	{  61,  1, 10}, /* 34.31 */
	{  58,  1, 11}, /* 34.44 */
	{  63,  0, 27}, /* 34.45 */
	{  62,  1, 10}, /* 34.88 */
	{  63,  1, 10}, /* 35.44 */
	{  60,  1, 11}, /* 35.62 */
	{  52,  1, 14}, /* 35.75 */
	{  62,  0, 29}, /* 35.84 */
	{  61,  1, 11}, /* 36.22 */
	{  58,  1, 12}, /* 36.25 */
	{  63,  0, 29}, /* 36.42 */
	{  62,  1, 11}, /* 36.81 */
	{  63,  1, 11}, /* 37.41 */
	{  60,  1, 12}, /* 37.50 */
	{  62,  0, 31}, /* 37.78 */
	{  61,  1, 12}, /* 38.12 */
	{  63,  0, 31}, /* 38.39 */
	{  60,  0, 33}, /* 38.44 */
	{  62,  1, 12}, /* 38.75 */
	{  61,  0, 33}, /* 39.08 */
	{  63,  1, 12}, /* 39.38 */
	{  55,  1, 15}, /* 39.53 */
	{  62,  0, 33}, /* 39.72 */
	{  61,  1, 13}, /* 40.03 */
	{  63,  0, 33}, /* 40.36 */
	{  62,  1, 13}, /* 40.69 */
	{  45,  1, 21}, /* 40.78 */
	{  57,  1, 15}, /* 40.97 */
	{  63,  1, 13}, /* 41.34 */
	{  53,  1, 17}, /* 41.41 */
	{  62,  0, 35}, /* 41.66 */
	{  61,  1, 14}, /* 41.94 */
	{  63,  0, 35}, /* 42.33 */
	{  62,  1, 14}, /* 42.62 */
	{  57,  1, 16}, /* 42.75 */
	{  61,  0, 37}, /* 42.89 */
	{  63,  1, 14}, /* 43.31 */
	{  62,  0, 37}, /* 43.59 */
	{  61,  1, 15}, /* 43.84 */
	{  54,  1, 18}, /* 43.88 */
	{  63,  0, 37}, /* 44.30 */
	{  62,  1, 15}, /* 44.56 */
	{  61,  0, 39}, /* 44.80 */
	{  60,  1, 16}, /* 45.00 */
	{  63,  1, 15}, /* 45.28 */
	{  58,  1, 17}, /* 45.31 */
	{  62,  0, 39}, /* 45.53 */
	{  61,  1, 16}, /* 45.75 */
	{  63,  0, 39}, /* 46.27 */
	{  62,  1, 16}, /* 46.50 */
	{  61,  0, 41}, /* 46.70 */
	{  60,  1, 17}, /* 46.88 */
	{  63,  1, 16}, /* 47.25 */
	{  62,  0, 41}, /* 47.47 */
	{  61,  1, 17}, /* 47.66 */
	{  60,  0, 43}, /* 47.81 */
	{  59,  1, 18}, /* 47.94 */
	{  63,  0, 41}, /* 48.23 */
	{  62,  1, 17}, /* 48.44 */
	{  61,  0, 43}, /* 48.61 */
	{  63,  1, 17}, /* 49.22 */
	{  62,  0, 43}, /* 49.41 */
	{  61,  1, 18}, /* 49.56 */
	{  60,  0, 45}, /* 49.69 */
	{  59,  1, 19}, /* 49.78 */
	{  63,  0, 43}, /* 50.20 */
	{  62,  1, 18}, /* 50.38 */
	{  61,  0, 45}, /* 50.52 */
	{  63,  1, 18}, /* 51.19 */
	{  62,  0, 45}, /* 51.34 */
	{  61,  1, 19}, /* 51.47 */
	{  60,  0, 47}, /* 51.56 */
	{  59,  1, 20}, /* 51.62 */
	{  63,  0, 45}, /* 52.17 */
	{  62,  1, 19}, /* 52.31 */
	{  61,  0, 47}, /* 52.42 */
	{  63,  1, 19}, /* 53.16 */
	{  62,  0, 47}, /* 53.28 */
	{  61,  1, 20}, /* 53.38 */
	{  60,  0, 49}, /* 53.44 */
	{  63,  0, 47}, /* 54.14 */
	{  62,  1, 20}, /* 54.25 */
	{  61,  0, 49}, /* 54.33 */
	{  60,  1, 21}, /* 54.38 */
	{  63,  1, 20}, /* 55.12 */
	{  62,  0, 49}, /* 55.22 */
	{  61,  1, 21}, /* 55.28 */
	{  59,  1, 22}, /* 55.31 */
	{  63,  0, 49}, /* 56.11 */
	{  62,  1, 21}, /* 56.19 */
	{  61,  0, 51}, /* 56.23 */
	{  60,  1, 22}, /* 56.25 */
	{  63,  1, 21}, /* 57.09 */
	{  62,  0, 51}, /* 57.16 */
	{  61,  1, 22}, /* 57.19 */
	{  56,  1, 25}, /* 57.75 */
	{  63,  0, 51}, /* 58.08 */
	{  62,  1, 22}, /* 58.12 */
	{  61,  0, 53}, /* 58.14 */
	{  57,  1, 25}, /* 58.78 */
	{  63,  1, 22}, /* 59.06 */
	{  61,  1, 23}, /* 59.09 */
	{  56,  1, 26}, /* 59.50 */
	{  58,  1, 25}, /* 59.81 */
	{  63,  0, 53}, /* 60.05 */
	{  62,  1, 23}, /* 60.06 */
	{  57,  1, 26}, /* 60.56 */
	{  59,  1, 25}, /* 60.84 */
	{  63,  1, 23}, /* 61.03 */
	{  56,  1, 27}, /* 61.25 */
	{  58,  1, 26}, /* 61.62 */
	{  60,  1, 25}, /* 61.88 */
	{  62,  1, 24}, /* 62.00 */
	{  63,  0, 55}, /* 62.02 */
	{  57,  1, 27}, /* 62.34 */
	{  61,  1, 25}, /* 62.91 */
	{  63,  1, 24}, /* 63.00 */
	{  57,  0, 63}, /* 63.23 */
	{  58,  1, 27}, /* 63.44 */
	{  60,  1, 26}, /* 63.75 */
	{  62,  1, 25}, /* 63.94 */
	{  63,  0, 57}, /* 63.98 */
	{  59,  1, 27}, /* 64.53 */
	{  61,  1, 26}, /* 64.81 */
};


static int mt9p031_read_reg(struct i2c_client *client, unsigned short reg)
{
	int err = 0;
	struct i2c_msg msg[1];
	unsigned char data[2];
	unsigned short val = 0;

	if (!client->adapter) {
		err = -ENODEV;
		return err;
	}else {
		/* TODO: addr should be set up where else client->addr */
		msg->addr = MT9P031_I2C_ADDR;
		msg->flags = 0;
		msg->len = I2C_ONE_BYTE_TRANSFER;
		msg->buf = data;
		data[0] = reg;
		err = i2c_transfer(client->adapter, msg, 1);
		if (err >= 0) {
			msg->flags = I2C_M_RD;
			msg->len = I2C_TWO_BYTE_TRANSFER;	/* 2 byte read */
			msg->buf = data;
			err = i2c_transfer(client->adapter, msg, 1);
			if (err >= 0) {
				val = ((data[0] & I2C_TXRX_DATA_MASK)
					<< I2C_TXRX_DATA_SHIFT)
				    | (data[1] & I2C_TXRX_DATA_MASK);
			}
		}
	}
	return (int)(0x0000ffff & val);
}



static int mt9p031_write_reg(struct i2c_client *client, unsigned short reg, unsigned short val)
{
	int err = 0;
	int trycnt = 0;

	struct i2c_msg msg[1];
	unsigned char data[3];
	err = -1;

	v4l_dbg(1, debug, client, "mt9p031_write_reg reg=0x%x, val=0x%x\n", reg, val);

	while ((err < 0) && (trycnt < I2C_RETRY_COUNT)) {
		trycnt++;
		if (!client->adapter) {
			err = -ENODEV;
		} else {
			/* TODO:addr should be set up where else client->addr*/ 
			msg->addr = MT9P031_I2C_ADDR;
			msg->flags = 0;
			msg->len = I2C_THREE_BYTE_TRANSFER;
			msg->buf = data;
			data[0] = reg;
			data[1] = (val & I2C_TXRX_DATA_MASK_UPPER) >> I2C_TXRX_DATA_SHIFT;
			data[2] = (val & I2C_TXRX_DATA_MASK);
			err = i2c_transfer(client->adapter, msg, 1);

			if ( err < 0 ){
				printk(KERN_INFO "The i2c write 0x%x failed\n", reg);
			}
		}
	}

	if (err < 0) {
		printk(KERN_INFO "\n I2C write failed.");
	}

	return err;

}

/*
 * mt9p031_write_regs : Initializes a list of registers
 *		if token is TOK_TERM, then entire write operation terminates
 *		if token is TOK_DELAY, then a delay of 'val' msec is introduced
 *		if token is TOK_SKIP, then the register write is skipped
 *		if token is TOK_WRITE, then the register write is performed
 *
 * reglist - list of registers to be written
 * Returns zero if successful, or non-zero otherwise.
 */
static int mt9p031_write_regs(struct i2c_client *client,
			      const struct mt9p031_reg reglist[])
{
	int err;
	const struct mt9p031_reg *next = reglist;

	for (; next->token != TOK_TERM; next++) {
		if (next->token == TOK_DELAY) {
			msleep(next->val);
			continue;
		}

		if (next->token == TOK_SKIP)
			continue;

		err = mt9p031_write_reg(client, next->reg, next->val);
		if (err < 0) {
			v4l_err(client, "Write failed. Err[%d]\n", err);
			return err;
		}
	}
	return 0;
}

/*
 * mt9p031_get_current_std:
 * Returns the current standard
 */
static enum mt9p031_std mt9p031_get_current_std(struct mt9p031_decoder *decoder)
{
	return decoder->current_std;
}

/*
 * Configure the mt9p031 with the current register settings
 * Returns zero if successful, or non-zero otherwise.
 */
static int mt9p031_configure(struct mt9p031_decoder *decoder)
{
	int err;

	err = mt9p031_write_regs(decoder->client, mt9p031_reg_list);
	switch (decoder->current_std)  {
	case  MT9P031_STD_VGA:
		printk(KERN_INFO "VGA \n");
		err |= mt9p031_write_regs(decoder->client, mt9p031_vga_reg);
		break;
	case  MT9P031_STD_XGA:
		printk(KERN_INFO "XGA \n");
		err |= mt9p031_write_regs(decoder->client, mt9p031_xga_reg);
		break;
	case  MT9P031_STD_720P:
		printk(KERN_INFO "720P \n");
		err |= mt9p031_write_regs(decoder->client, mt9p031_720p_reg);
		break;
	case  MT9P031_STD_1080P:
		printk(KERN_INFO "1080P \n");
		err |= mt9p031_write_regs(decoder->client, mt9p031_1080p_reg);
		break;
	case  MT9P031_STD_2592X1944:
		printk(KERN_INFO "2592X1944 Maximum Resolution \n");
		err |= mt9p031_write_regs(decoder->client, mt9p031_2592X1944_reg);
		break;
	default:
		v4l_err(decoder->client, "MT9P031: Invalid standard: %d\n",decoder->current_std);
	}

	if (err)
		return err;

	return 0;
}


/*
 * ioctl_enum_framesizes - V4L2 sensor if handler for vidioc_int_enum_framesizes
 * @s: pointer to standard V4L2 device structure
 * @frms: pointer to standard V4L2 framesizes enumeration structure
 *
 * Returns possible framesizes depending on choosen pixel format
 */
static int ioctl_enum_framesizes(struct v4l2_int_device *s, struct v4l2_frmsizeenum *frms)
{
	struct mt9p031_decoder *decoder = s->priv;
	int ifmt;

	for (ifmt = 0; ifmt < decoder->num_fmts; ifmt++) {
		if (frms->pixel_format == decoder->fmt_list[ifmt].pixelformat)
			break;
	}
	/* Is requested pixelformat not found on sensor? */
	if (ifmt == decoder->num_fmts)
		return -EINVAL;

	/* Do we already reached all discrete framesizes? */
	if (frms->index >= decoder->num_stds)
		return -EINVAL;

	frms->type = V4L2_FRMSIZE_TYPE_DISCRETE;
	frms->discrete.width = decoder->std_list[frms->index].width;
	frms->discrete.height = decoder->std_list[frms->index].height;

	return 0;

}

static int ioctl_enum_frameintervals(struct v4l2_int_device *s,struct v4l2_frmivalenum *frmi)
{
	struct mt9p031_decoder *decoder = s->priv;
	int ifmt;

	if (frmi->index >= 1)
		return -EINVAL;

	for (ifmt = 0; ifmt < decoder->num_fmts; ifmt++) {
		if (frmi->pixel_format == decoder->fmt_list[ifmt].pixelformat)
			break;
	}
	/* Is requested pixelformat not found on sensor? */
	if (ifmt == decoder->num_fmts)
		return -EINVAL;

	if (frmi->index >= ARRAY_SIZE(mt9p031_frameintervals))
		return -EINVAL;

	switch ( frmi->width) {
		case 640:
			ifmt = 0;
			break;
		case 1024:
			ifmt = 1;
			break;
		case 1280:
			ifmt = 2;
			break;
		case 1920:
			ifmt = 3;
			break;
		case 2592:
			ifmt = 4;
			break;
		default:
			return -EINVAL;
	}
	frmi->type = V4L2_FRMSIZE_TYPE_DISCRETE;
	frmi->discrete.numerator = mt9p031_std_list[ifmt].standard.frameperiod.numerator;
	frmi->discrete.denominator = mt9p031_std_list[ifmt].standard.frameperiod.denominator;

	return 0;
}

/*
 * Detect if an mt9p031 is present, and if so which revision.
 * A device is considered to be detected if the chip ID (LSB and MSB)
 * registers match the expected values.
 * Any value of the rom version register is accepted.
 * Returns ENODEV error number if no device is detected, or zero
 * if a device is detected.
 */
static int mt9p031_detect(struct mt9p031_decoder *decoder)
{
	unsigned short val=0;

	val = mt9p031_read_reg(decoder->client, 0xFF);

	v4l_dbg(1, debug, decoder->client, "Detected Chip Version 0x%x\n", val);

	if (MT9P031_CHIP_VERSION_VALUE != val) {
		/* We didn't read the values we expected, so this must not be
		 * MT9P031.
		 */
		v4l_err(decoder->client,
			"Chip id mismatch: Read 0x%x, Expecting 0x%x\n", val, MT9P031_CHIP_VERSION_VALUE);
		return -ENODEV;
	}

	decoder->ver = val;
	decoder->state = STATE_DETECTED;

	v4l_info(decoder->client,
			"%s found at 0x%x (%s)\n", decoder->client->name,
			decoder->client->addr << 1,
			decoder->client->adapter->name);

	return 0;
}

/*
 * Following are decoder interface functions implemented by
 * mt9p031 decoder driver.
 */

/*
 * ioctl_querystd - V4L2 decoder interface handler for VIDIOC_QUERYSTD ioctl
 * @s: pointer to standard V4L2 device structure
 * @std_id: standard V4L2 std_id ioctl enum
 *
 * Returns the current standard detected by mt9p031. If no active input is
 * detected, returns -EINVAL
 */
static int ioctl_querystd(struct v4l2_int_device *s, v4l2_std_id *std_id)
{
	struct mt9p031_decoder *decoder = s->priv;
	enum mt9p031_std current_std;

	if (std_id == NULL)
		return -EINVAL;

	/* get the current standard */
	current_std = mt9p031_get_current_std(decoder);
	if (current_std == MT9P031_IMAGE_STD_INVALID)
		return -EINVAL;

	*std_id = decoder->std_list[current_std].standard.id;

	v4l_dbg(1, debug, decoder->client, "Current STD: %s",
			decoder->std_list[current_std].standard.name);
	return 0;
}

/*
 * ioctl_s_std - V4L2 decoder interface handler for VIDIOC_S_STD ioctl
 * @s: pointer to standard V4L2 device structure
 * @std_id: standard V4L2 v4l2_std_id ioctl enum
 *
 * If std_id is supported, sets the requested standard. Otherwise, returns
 * -EINVAL
 */
static int ioctl_s_std(struct v4l2_int_device *s, v4l2_std_id *std_id)
{
	struct mt9p031_decoder *decoder = s->priv;
	int err, i;

	if (std_id == NULL)
		return -EINVAL;

	for (i = 0; i < decoder->num_stds; i++)
		if (*std_id & decoder->std_list[i].standard.id)
			break;

	if ((i == decoder->num_stds) || (i == MT9P031_STD_INVALID))
		return -EINVAL;

	decoder->current_std = decoder->std_list[i].standard.index;

	err = mt9p031_configure(decoder);

	if (err)
		return err;

	//mt9p031_reg_list[REG_VIDEO_STD].val = decoder->std_list[i].video_std;

	v4l_dbg(1, debug, decoder->client, "Standard set to: %s",
			decoder->std_list[i].standard.name);
	return 0;
}

/*
 * ioctl_s_routing - V4L2 decoder interface handler for VIDIOC_S_INPUT ioctl
 * @s: pointer to standard V4L2 device structure
 * @index: number of the input
 *
 * If index is valid, selects the requested input. Otherwise, returns -EINVAL if
 * the input is not supported or there is no active signal present in the
 * selected input.
 */
static int ioctl_s_routing(struct v4l2_int_device *s,
				struct v4l2_routing *route)
{
	return 0;
}

/*
 * ioctl_queryctrl - V4L2 decoder interface handler for VIDIOC_QUERYCTRL ioctl
 * @s: pointer to standard V4L2 device structure
 * @qctrl: standard V4L2 v4l2_queryctrl structure
 *
 * If the requested control is supported, returns the control information.
 * Otherwise, returns -EINVAL if the control is not supported.
 */
static int ioctl_queryctrl(struct v4l2_int_device *s, struct v4l2_queryctrl *qctrl)
{
	struct mt9p031_decoder *decoder = s->priv;
	int err = -EINVAL;

	if (qctrl == NULL)
		return err;

	switch (qctrl->id) {
	case V4L2_CID_EXPOSURE:
		err = v4l2_ctrl_query_fill(qctrl, 0, 65535, 1,
			decoder->current_exposure);
	case V4L2_CID_GAIN:
		err = v4l2_ctrl_query_fill(qctrl, 0, 255, 1,
			decoder->current_gain);
		break;
	default:
		v4l_err(decoder->client,
			"invalid control id %d\n", qctrl->id);
		return err;
	}

	v4l_dbg(1, debug, decoder->client,
			"Query Control: %s : Min - %d, Max - %d, Def - %d",
			qctrl->name,
			qctrl->minimum,
			qctrl->maximum,
			qctrl->default_value);

	return err;
}

/*
 * ioctl_g_ctrl - V4L2 decoder interface handler for VIDIOC_G_CTRL ioctl
 * @s: pointer to standard V4L2 device structure
 * @ctrl: pointer to v4l2_control structure
 *
 * If the requested control is supported, returns the control's current
 * value from the decoder. Otherwise, returns -EINVAL if the control is not
 * supported.
 */
static int ioctl_g_ctrl(struct v4l2_int_device *s, struct v4l2_control *ctrl)
{
	struct mt9p031_decoder *decoder = s->priv;

	if (ctrl == NULL)
		return -EINVAL;

	switch (ctrl->id) {
	case V4L2_CID_EXPOSURE:
		ctrl->value = decoder->current_exposure;
		break;
	case V4L2_CID_GAIN:
		ctrl->value = decoder->current_gain;
		break;
	default:
		v4l_err(decoder->client,
			"invalid control id %d\n", ctrl->id);
		return -EINVAL;
	}

	v4l_dbg(1, debug, decoder->client,
			"Get Control: ID - %d - %d",
			ctrl->id, ctrl->value);
	return 0;
}

/*
 * ioctl_s_ctrl - V4L2 decoder interface handler for VIDIOC_S_CTRL ioctl
 * @s: pointer to standard V4L2 device structure
 * @ctrl: pointer to v4l2_control structure
 *
 * If the requested control is supported, sets the control's current
 * value in HW. Otherwise, returns -EINVAL if the control is not supported.
 */
static int ioctl_s_ctrl(struct v4l2_int_device *s, struct v4l2_control *ctrl)
{
	struct mt9p031_decoder *decoder = s->priv;
	struct mt9p031_gain new_gain;
	int err = -EINVAL, value;

	if (ctrl == NULL)
		return err;

	value = (unsigned short) ctrl->value;

	switch (ctrl->id) {
	case V4L2_CID_EXPOSURE:
		err = mt9p031_write_reg(decoder->client,
			MT9P031_SHUTTER_WIDTH,
			value & MT9P031_SHUTTER_WIDTH_LOWER_MASK);
		err = err | mt9p031_write_reg(decoder->client,
			MT9P031_SHUTTER_WIDTH_UPPER,
			value >> MT9P031_SHUTTER_WIDTH_UPPER_SHIFT);
		break;
	case V4L2_CID_GAIN:
		if (value > 255) {
			value = 255;
		}
		new_gain = mt9p031_gain_table[value];
		err = mt9p031_write_reg(decoder->client,
			MT9P031_GLOBAL_GAIN,
			new_gain.analog
			+ (new_gain.analog_mult << 6)
			+ (new_gain.digital << 8));
		break;
	default:
		v4l_err(decoder->client,
			"invalid control id %d\n", ctrl->id);
		return err;
	}

	v4l_dbg(1, debug, decoder->client,
			"Set Control: ID - %d - %d",
			ctrl->id, ctrl->value);

	return err;
}

/*
 * ioctl_enum_fmt_cap - Implement the CAPTURE buffer VIDIOC_ENUM_FMT ioctl
 * @s: pointer to standard V4L2 device structure
 * @fmt: standard V4L2 VIDIOC_ENUM_FMT ioctl structure
 *
 * Implement the VIDIOC_ENUM_FMT ioctl to enumerate supported formats
 */
static int ioctl_enum_fmt_cap(struct v4l2_int_device *s, struct v4l2_fmtdesc *fmt)
{
	struct mt9p031_decoder *decoder = s->priv;
	int index;

	if (fmt == NULL)
		return -EINVAL;

	index = fmt->index;
	if ((index >= decoder->num_fmts) || (index < 0))
		return -EINVAL;	/* Index out of bound */

	if (fmt->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;	/* only capture is supported */

	memcpy(fmt, &decoder->fmt_list[index],
		sizeof(struct v4l2_fmtdesc));

	v4l_dbg(1, debug, decoder->client,
			"Current FMT: index - %d (%s)",
			decoder->fmt_list[index].index,
			decoder->fmt_list[index].description);
	return 0;
}

/*
 * ioctl_try_fmt_cap - Implement the CAPTURE buffer VIDIOC_TRY_FMT ioctl
 * @s: pointer to standard V4L2 device structure
 * @f: pointer to standard V4L2 VIDIOC_TRY_FMT ioctl structure
 *
 * Implement the VIDIOC_TRY_FMT ioctl for the CAPTURE buffer type. This
 * ioctl is used to negotiate the image capture size and pixel format
 * without actually making it take effect.
 */
static int ioctl_try_fmt_cap(struct v4l2_int_device *s, struct v4l2_format *f)
{
	struct mt9p031_decoder *decoder = s->priv;
	int ifmt;
	struct v4l2_pix_format *pix;
	enum mt9p031_std current_std;

	if (f == NULL)
		return -EINVAL;

	if (f->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		f->type = V4L2_BUF_TYPE_VIDEO_CAPTURE;

	pix = &f->fmt.pix;

	/* Calculate height and width based on current standard */
	current_std = mt9p031_get_current_std(decoder);
	if (current_std == MT9P031_STD_INVALID)
		return -EINVAL;

	//decoder->current_std = current_std;
	//pix->width = decoder->std_list[current_std].width;
	//pix->height = decoder->std_list[current_std].height;
	//printk(KERN_INFO "The number of formats are %d", decoder->num_fmts);

	for (ifmt = 0; ifmt < decoder->num_fmts; ifmt++) {
		if (pix->pixelformat ==	decoder->fmt_list[ifmt].pixelformat)
			break;
	}
	if (ifmt == decoder->num_fmts) {
		ifmt = 0;	/* None of the format matched, select default */
		printk(KERN_INFO "Pixel Format didn't match");
	}

	pix->pixelformat = decoder->fmt_list[ifmt].pixelformat;

	for (ifmt = 0; ifmt < decoder->num_stds; ifmt++) {
		if (pix->width == decoder->std_list[ifmt].width)
			break;
	}
	if (ifmt == decoder->num_stds) {
		ifmt = 0;	/* None of the format matched, select default */
		printk(KERN_INFO "Width didn't match");
	}

	pix->width = decoder->std_list[ifmt].width;
	pix->height = decoder->std_list[ifmt].height;

	pix->field = V4L2_FIELD_NONE;
	pix->bytesperline = pix->width * 2;
	pix->sizeimage = pix->bytesperline * pix->height;
	pix->colorspace = V4L2_COLORSPACE_SRGB;
	pix->priv = 0;
	
	decoder->pix = *pix;
	decoder->current_std=decoder->std_list[ifmt].video_std;

	v4l_dbg(1, debug, decoder->client,
			"Try FMT: pixelformat - %s, bytesperline - %d"
			"Width - %d, Height - %d",
			decoder->fmt_list[ifmt].description, pix->bytesperline,
			pix->width, pix->height);
	return 0;
}

/*
 * ioctl_s_fmt_cap - V4L2 decoder interface handler for VIDIOC_S_FMT ioctl
 * @s: pointer to standard V4L2 device structure
 * @f: pointer to standard V4L2 VIDIOC_S_FMT ioctl structure
 *
 * If the requested format is supported, configures the HW to use that
 * format, returns error code if format not supported or HW can't be
 * correctly configured.
 */
static int ioctl_s_fmt_cap(struct v4l2_int_device *s, struct v4l2_format *f)
{
	struct mt9p031_decoder *decoder = s->priv;
	struct v4l2_pix_format *pix;
	int rval;

	if (f == NULL)
		return -EINVAL;

	if (f->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;	/* only capture is supported */

	pix = &f->fmt.pix;
	rval = ioctl_try_fmt_cap(s, f);
	if (rval)
		return rval;

	decoder->pix = *pix;

	return rval;
}

/*
 * ioctl_g_fmt_cap - V4L2 decoder interface handler for ioctl_g_fmt_cap
 * @s: pointer to standard V4L2 device structure
 * @f: pointer to standard V4L2 v4l2_format structure
 *
 * Returns the decoder's current pixel format in the v4l2_format
 * parameter.
 */
static int ioctl_g_fmt_cap(struct v4l2_int_device *s, struct v4l2_format *f)
{
	struct mt9p031_decoder *decoder = s->priv;

	if (f == NULL)
		return -EINVAL;

	if (f->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;	/* only capture is supported */

	f->fmt.pix = decoder->pix;

	v4l_dbg(1, debug, decoder->client,
			"Current FMT: bytesperline - %d"
			"Width - %d, Height - %d",
			decoder->pix.bytesperline,
			decoder->pix.width, decoder->pix.height);
	return 0;
}

/*
 * ioctl_g_parm - V4L2 decoder interface handler for VIDIOC_G_PARM ioctl
 * @s: pointer to standard V4L2 device structure
 * @a: pointer to standard V4L2 VIDIOC_G_PARM ioctl structure
 *
 * Returns the decoder's video CAPTURE parameters.
 */
static int ioctl_g_parm(struct v4l2_int_device *s, struct v4l2_streamparm *a)
{
	struct mt9p031_decoder *decoder = s->priv;
	struct v4l2_captureparm *cparm;
	enum mt9p031_std current_std;

	if (a == NULL)
		return -EINVAL;

	if (a->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;	/* only capture is supported */

	memset(a, 0, sizeof(*a));
	a->type = V4L2_BUF_TYPE_VIDEO_CAPTURE;

	/* get the current standard */
	current_std = mt9p031_get_current_std(decoder);
	if (current_std == MT9P031_STD_INVALID)
		return -EINVAL;

	decoder->current_std = current_std;

	cparm = &a->parm.capture;
	cparm->capability = V4L2_CAP_TIMEPERFRAME;
	cparm->timeperframe =
		decoder->std_list[current_std].standard.frameperiod;

	return 0;
}

/*
 * ioctl_s_parm - V4L2 decoder interface handler for VIDIOC_S_PARM ioctl
 * @s: pointer to standard V4L2 device structure
 * @a: pointer to standard V4L2 VIDIOC_S_PARM ioctl structure
 *
 * Configures the decoder to use the input parameters, if possible. If
 * not possible, returns the appropriate error code.
 */
static int ioctl_s_parm(struct v4l2_int_device *s, struct v4l2_streamparm *a)
{
	struct mt9p031_decoder *decoder = s->priv;
	struct v4l2_fract *timeperframe;
	enum mt9p031_std current_std;

	if (a == NULL)
		return -EINVAL;

	if (a->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;	/* only capture is supported */

	timeperframe = &a->parm.capture.timeperframe;

	/* get the current standard */
	current_std = mt9p031_get_current_std(decoder);
	if (current_std == MT9P031_STD_INVALID)
		return -EINVAL;

	decoder->current_std = current_std;

	*timeperframe = decoder->std_list[current_std].standard.frameperiod;

	return 0;
}

/*
 * ioctl_g_ifparm - V4L2 decoder interface handler for vidioc_int_g_ifparm_num
 * @s: pointer to standard V4L2 device structure
 * @p: pointer to standard V4L2 vidioc_int_g_ifparm_num ioctl structure
 *
 * Gets slave interface parameters.
 * Calculates the required xclk value to support the requested
 * clock parameters in p. This value is returned in the p
 * parameter.
 */
static int ioctl_g_ifparm(struct v4l2_int_device *s, struct v4l2_ifparm *p)
{
	struct mt9p031_decoder *decoder = s->priv;
	int rval;

	if (p == NULL)
		return -EINVAL;

	if (NULL == decoder->pdata->ifparm)
		return -EINVAL;

	rval = decoder->pdata->ifparm(p);
	if (rval) {
		v4l_err(decoder->client, "g_ifparm.Err[%d]\n", rval);
		return rval;
	}

	/*We would like to have maximum pixel clock rate*/
	p->u.bt656.clock_curr = MT9P031_CLK_MAX;

	return 0;
}

/*
 * ioctl_g_priv - V4L2 decoder interface handler for vidioc_int_g_priv_num
 * @s: pointer to standard V4L2 device structure
 * @p: void pointer to hold decoder's private data address
 *
 * Returns device's (decoder's) private data area address in p parameter
 */
static int ioctl_g_priv(struct v4l2_int_device *s, void *p)
{
	struct mt9p031_decoder *decoder = s->priv;

	if (NULL == decoder->pdata->priv_data_set)
		return -EINVAL;

	return decoder->pdata->priv_data_set(p);
}

/*
 * ioctl_s_power - V4L2 decoder interface handler for vidioc_int_s_power_num
 * @s: pointer to standard V4L2 device structure
 * @on: power state to which device is to be set
 *
 * Sets devices power state to requrested state, if possible.
 */
static int ioctl_s_power(struct v4l2_int_device *s, enum v4l2_power on)
{
	struct mt9p031_decoder *decoder = s->priv;
	int err = 0;

	switch (on) {
	case V4L2_POWER_OFF:
		/* Disable mux for mtp031 data path */
		if (decoder->pdata->power_set)
			err |= decoder->pdata->power_set(s, on);
		break;

	case V4L2_POWER_STANDBY:
		if (decoder->pdata->power_set)
			err = decoder->pdata->power_set(s, on);
		break;

	case V4L2_POWER_ON:
		if (decoder->pdata->power_set) {
			err = decoder->pdata->power_set(s, on);
			if (err)
				return err;
		}

		if (decoder->state == STATE_NOT_DETECTED) {
			/* Detect the sensor is not already detected */
			err = mt9p031_detect(decoder);
			if (err) {
				v4l_err(decoder->client,
						"Unable to detect decoder\n");
				WARN_ON(1);
				return err;
			}
		}

		err = mt9p031_configure(decoder);
		if (err)
			return err;
		break;

	default:
		err = -ENODEV;
		break;
	}

	return err;
}

/*
 * ioctl_init - V4L2 decoder interface handler for VIDIOC_INT_INIT
 * @s: pointer to standard V4L2 device structure
 *
 * Initialize the decoder device (calls mt9p031_configure())
 */
static int ioctl_init(struct v4l2_int_device *s)
{
	struct mt9p031_decoder *decoder = s->priv;
	int err = 0;

	err = mt9p031_configure(decoder);

	return err;
}

/*
 * ioctl_dev_exit - V4L2 decoder interface handler for vidioc_int_dev_exit_num
 * @s: pointer to standard V4L2 device structure
 *
 * Delinitialise the dev. at slave detach. The complement of ioctl_dev_init.
 */
static int ioctl_dev_exit(struct v4l2_int_device *s)
{
	return 0;
}

/*
 * ioctl_dev_init - V4L2 decoder interface handler for vidioc_int_dev_init_num
 * @s: pointer to standard V4L2 device structure
 *
 * Initialise the device when slave attaches to the master. Returns 0 if
 * mt9p031 device could be found, otherwise returns appropriate error.
 */
static int ioctl_dev_init(struct v4l2_int_device *s)
{
	struct mt9p031_decoder *decoder = s->priv;
	int err;

	err = mt9p031_detect(decoder);
	if (err < 0) {
		v4l_err(decoder->client,
			"Unable to detect decoder\n");
		return err;
	}

	v4l_info(decoder->client,
		 "chip version 0x%.2x detected\n", decoder->ver);


	err |= mt9p031_configure(decoder);

	return 0;
}

static struct v4l2_int_ioctl_desc mt9p031_ioctl_desc[] = {
	{vidioc_int_dev_init_num, 	(v4l2_int_ioctl_func *) ioctl_dev_init},
	{vidioc_int_dev_exit_num, 	(v4l2_int_ioctl_func *) ioctl_dev_exit},
	{vidioc_int_s_power_num,  	(v4l2_int_ioctl_func *) ioctl_s_power},
	{vidioc_int_g_priv_num,   	(v4l2_int_ioctl_func *) ioctl_g_priv},
	{vidioc_int_g_ifparm_num, 	(v4l2_int_ioctl_func *) ioctl_g_ifparm},
	{vidioc_int_init_num,     	(v4l2_int_ioctl_func *) ioctl_init},
	{vidioc_int_enum_fmt_cap_num, 	(v4l2_int_ioctl_func *) ioctl_enum_fmt_cap},
	{vidioc_int_try_fmt_cap_num,	(v4l2_int_ioctl_func *) ioctl_try_fmt_cap},
	{vidioc_int_g_fmt_cap_num,	(v4l2_int_ioctl_func *) ioctl_g_fmt_cap},
	{vidioc_int_s_fmt_cap_num,	(v4l2_int_ioctl_func *) ioctl_s_fmt_cap},
	{vidioc_int_g_parm_num, 	(v4l2_int_ioctl_func *) ioctl_g_parm},
	{vidioc_int_s_parm_num, 	(v4l2_int_ioctl_func *) ioctl_s_parm},
	{vidioc_int_queryctrl_num,	(v4l2_int_ioctl_func *) ioctl_queryctrl},
	{vidioc_int_g_ctrl_num, 	(v4l2_int_ioctl_func *) ioctl_g_ctrl},
	{vidioc_int_s_ctrl_num, 	(v4l2_int_ioctl_func *) ioctl_s_ctrl},
	{vidioc_int_querystd_num, 	(v4l2_int_ioctl_func *) ioctl_querystd},
	{vidioc_int_s_std_num, 		(v4l2_int_ioctl_func *) ioctl_s_std},
	{vidioc_int_s_video_routing_num,(v4l2_int_ioctl_func *) ioctl_s_routing},
	{vidioc_int_enum_framesizes_num,(v4l2_int_ioctl_func *) ioctl_enum_framesizes},
	{vidioc_int_enum_frameintervals_num,(v4l2_int_ioctl_func*) ioctl_enum_frameintervals},
};


static struct v4l2_int_slave mt9p031_slave = {
	.ioctls = mt9p031_ioctl_desc,
	.num_ioctls = ARRAY_SIZE(mt9p031_ioctl_desc),
};

static struct mt9p031_decoder mt9p031_dev = {
	.state = STATE_NOT_DETECTED,

	.fmt_list = mt9p031_fmt_list,
	.num_fmts = ARRAY_SIZE(mt9p031_fmt_list),

	.pix = {
		.width = 640,
		.height = 480,
		.pixelformat = V4L2_PIX_FMT_SGRBG10,
		.field = V4L2_FIELD_NONE,
		.bytesperline = 640 * 2,
		.sizeimage = 640 * 2 * 480,
		.colorspace = V4L2_COLORSPACE_SRGB,
/*		.width = 2592,
		.height = 1944,
		.pixelformat = V4L2_PIX_FMT_SGRBG10,
		.field = V4L2_FIELD_NONE,
		.bytesperline = 2592 * 2,
		.sizeimage = 1944 * 2 * 1080,
		.colorspace = V4L2_COLORSPACE_SRGB,*/
		},

	.current_std = MT9P031_STD_VGA,
	.std_list = mt9p031_std_list,
	.num_stds = ARRAY_SIZE(mt9p031_std_list),

	.current_gain = 32,
	.current_exposure = 1000,
};

static struct v4l2_int_device mt9p031_int_device = {
	.module = THIS_MODULE,
	.name = MT9P031_MODULE_NAME,
	.priv = &mt9p031_dev,
	.type = v4l2_int_type_slave,
	.u = {
	      .slave = &mt9p031_slave,
	      },
};

/*
 * mt9p031_probe - decoder driver i2c probe handler
 * @client: i2c driver client device structure
 *
 * Register decoder as an i2c client device and V4L2
 * device.
 */
static int mt9p031_probe(struct i2c_client *client, const struct i2c_device_id *id)
{
	struct mt9p031_decoder *decoder = &mt9p031_dev;
	int err;
	/* Check if the adapter supports the needed features */
	if (!i2c_check_functionality(client->adapter, I2C_FUNC_SMBUS_BYTE_DATA))
		return -EIO;

	decoder->pdata = client->dev.platform_data;
	if (!decoder->pdata) {
		v4l_err(client, "No platform data!!\n");
		return -ENODEV;
	}
	/*
	 * Save the id data, required for power up sequence
	 */
	decoder->id = (struct i2c_device_id *)id;
	/* Attach to Master */
	strcpy(mt9p031_int_device.u.slave->attach_to, decoder->pdata->master);
	decoder->v4l2_int_device = &mt9p031_int_device;
	decoder->client = client;
	i2c_set_clientdata(client, decoder);

	/* Register with V4L2 layer as slave device */
	err = v4l2_int_device_register(decoder->v4l2_int_device);
	if (err) {
		i2c_set_clientdata(client, NULL);
		v4l_err(client,
			"Unable to register to v4l2. Err[%d]\n", err);

	} else
		v4l_info(client, "Registered to v4l2 master %s!!\n",
				decoder->pdata->master);

	return 0;
}

/*
 * mt9p031_remove - decoder driver i2c remove handler
 * @client: i2c driver client device structure
 *
 * Unregister decoder as an i2c client device and V4L2
 * device. Complement of mt9p031_probe().
 */
static int __exit mt9p031_remove(struct i2c_client *client)
{
	struct mt9p031_decoder *decoder = i2c_get_clientdata(client);

	if (!client->adapter)
		return -ENODEV;	/* our client isn't attached */

	v4l2_int_device_unregister(decoder->v4l2_int_device);
	i2c_set_clientdata(client, NULL);

	return 0;
}
/*
 * I2C Device Table -
 *
 * name - Name of the actual device/chip.
 * driver_data - Driver data
 */
static const struct i2c_device_id mt9p031_id[] = {
	{"mt9p031", 0},
	{},
};

MODULE_DEVICE_TABLE(i2c, mt9p031_id);

static struct i2c_driver mt9p031_i2c_driver = {
	.driver = {
		   .name = MT9P031_MODULE_NAME,
		   .owner = THIS_MODULE,
		   },
	.probe = mt9p031_probe,
	.remove = __exit_p(mt9p031_remove),
	.id_table = mt9p031_id,
};

/*
 * mt9p031_init
 *
 * Module init function
 */
static int __init mt9p031_init(void)
{
	int err;

	printk( KERN_INFO "Inserting mt9p031 camera module\n");

	err=i2c_add_driver(&mt9p031_i2c_driver);
	return err;
}

/*
 * mt9p031_cleanup
 *
 * Module exit function
 */
static void __exit mt9p031_cleanup(void)
{
	printk( KERN_INFO "\n Removing mt9p031 module");
	i2c_del_driver(&mt9p031_i2c_driver);
}

module_init(mt9p031_init);
module_exit(mt9p031_cleanup);

MODULE_AUTHOR("Colin Foe-Parker <foeparker.colin@gmail.com");
MODULE_DESCRIPTION("MT9P031 linux decoder driver");
MODULE_LICENSE("GPL");

